package cn.com.ty.lift.business.capital.service;

import cn.com.ty.lift.business.capital.dao.entity.SignIn;
import cn.com.ty.lift.business.capital.dao.entity.model.request.AttendanceListRequest;
import cn.com.ty.lift.business.capital.dao.entity.model.request.SignInRequest;
import cn.com.ty.lift.business.capital.dao.entity.model.response.SignInResponse;
import cn.com.ty.lift.business.capital.dao.mapper.SignInMapper;
import cn.com.ty.lift.business.project.dao.entity.model.response.ProjectUserResponse;
import cn.com.ty.lift.business.project.service.ProjectUserService;
import cn.com.ty.lift.common.constants.CommonEnum;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @author bieao
 * @date 2020/2/5
 * @description 大修签到业务层
 */
@Service
public class SignInService extends ServiceImpl<SignInMapper, SignIn> {

     @Resource
     private ProjectUserService projectUserService;
    /**
     * @param entry 用户签到信息
     * @return 是否成功
     * @description 大修签到
     * @date 2020/2/5 12:42 下午
     */
    public boolean add(SignIn entry) {
        return save(entry);
    }


   /**
    * @param request 公司ID，用户ID
    * @return 考勤人员列表
    * @description 考勤人员列表
    * @date 2020/4/15 2:52 下午
    */
    public List<ProjectUserResponse> attendanceList(AttendanceListRequest request) {
        //查询项目组成员列表
        List<ProjectUserResponse> userList = projectUserService.getUserList(request.getProjectId(), request.getMtCompanyId());

        //查询所有项目组成员考勤记录
        QueryWrapper<SignIn> queryWrapper = new QueryWrapper<>();
        LambdaQueryWrapper<SignIn> lambdaQueryWrapper = queryWrapper.lambda();
        lambdaQueryWrapper.eq(SignIn::getMtCompanyId, request.getMtCompanyId());
        lambdaQueryWrapper.eq(SignIn::getProjectId, request.getProjectId());
        //考勤记录列表
        ArrayList<SignIn> recordList = (ArrayList<SignIn>) list(lambdaQueryWrapper);

        List<ProjectUserResponse> responseList = userList.stream().filter(user -> (
                CommonEnum.DefaultRole.MTWORK.getCode().equals(user.getUserRole())
        )).collect(Collectors.toList());

        Optional<ProjectUserResponse> first = responseList.stream().filter(entry -> entry.getUserId().equals(request.getUserId())).findFirst();
        first.ifPresent(response -> {
            String monitor = response.getIsMonitor();
            //当前用户非维保班长的考勤记录
            if ("0".equals(monitor)) {
                recordList.removeIf(entry -> !entry.getUserId().equals(request.getUserId()));
            }
            //组装所有项目成员的考勤记录
            for (ProjectUserResponse entry : responseList) {
                List<SignIn> list = new ArrayList<>();
                for (SignIn signIn : recordList) {
                    if (entry.getUserId().equals(signIn.getUserId())) {
                        list.add(signIn);
                        entry.setAttendanceRecord(list);
                    }
                }
            }
        });
        return responseList;
    }

    /**
     * @param request 大修签到查询条件
     * @return RestResponse 结果集
     * @description 条件查询大修项目签到列表
     * @date 2020/2/5 12:58 下午
     */
    public IPage<SignInResponse> list(SignInRequest request) {
        IPage<SignInResponse> page = new Page<>(request.getPageNum(), request.getPageSize());
        return baseMapper.findByCondition(page, request);
    }
}
